@@ -98,6 +98,8 @@ struct lttng_session *lttng_session_create(void)
 	kref_init(&metadata_cache->refcount);
 	session->metadata_cache = metadata_cache;
 	INIT_LIST_HEAD(&metadata_cache->metadata_stream);
+	memcpy(&metadata_cache->uuid, &session->uuid,
+		sizeof(metadata_cache->uuid));
 	list_add(&session->list, &sessions);
 	mutex_unlock(&sessions_mutex);
 	return session;
@@ -597,16 +599,20 @@ int lttng_metadata_output_channel(struct lttng_metadata_stream *stream,
 
 	/*
 	 * Ensure we support mutiple get_next / put sequences followed
-	 * by put_next.
+	 * by put_next. The metadata stream lock internally protects
+	 * reading the metadata cache. It can indeed be read
+	 * concurrently by "get_next_subbuf" and "flush" operations on
+	 * the buffer invoked by different processes.
 	 */
+	mutex_lock(&stream->lock);
 	WARN_ON(stream->metadata_in < stream->metadata_out);
 	if (stream->metadata_in != stream->metadata_out)
-		return 0;
+		goto end;
 
 	len = stream->metadata_cache->metadata_written -
 		stream->metadata_in;
 	if (!len)
-		return 0;
+		goto end;
 	reserve_len = min_t(size_t,
 			stream->transport->ops.packet_avail_size(chan),
 			len);
@@ -628,6 +634,7 @@ int lttng_metadata_output_channel(struct lttng_metadata_stream *stream,
 	ret = reserve_len;
 
 end:
+	mutex_unlock(&stream->lock);
 	return ret;
 }
 
@@ -1255,6 +1262,10 @@ static int __init lttng_events_init(void)
 {
 	int ret;
 
+	ret = wrapper_lttng_fixup_sig(THIS_MODULE);
+	if (ret)
+		return ret;
+
 	ret = lttng_tracepoint_init();
 	if (ret)
 		return ret;
